/*
 * Syncdaemon API
 *
 * Authors: Rodrigo Moya <rodrigo.moya@canonical.com>
 *
 * Copyright 2010 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"
#include "syncdaemon-folders-interface.h"
#include "utils.h"

G_DEFINE_TYPE(SyncdaemonFoldersInterface, syncdaemon_folders_interface, SYNCDAEMON_TYPE_INTERFACE)

struct _SyncdaemonFoldersInterfacePrivate {
	GObject *proxy;
};

static void
syncdaemon_folders_interface_finalize (GObject *object)
{
	SyncdaemonFoldersInterface *interface = SYNCDAEMON_FOLDERS_INTERFACE (object);

	if (interface->priv != NULL) {
		g_free (interface->priv);
	}

	G_OBJECT_CLASS (syncdaemon_folders_interface_parent_class)->finalize (object);
}

static void
syncdaemon_folders_interface_class_init (SyncdaemonFoldersInterfaceClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	object_class->finalize = syncdaemon_folders_interface_finalize;
}

static void
folder_created_cb (DBusGProxy *proxy, GHashTable *folder_info, gpointer user_data)
{
	SyncdaemonDaemon *daemon = NULL;
	SyncdaemonFoldersInterface *interface = SYNCDAEMON_FOLDERS_INTERFACE (user_data);

	g_object_get (G_OBJECT (interface), "daemon", &daemon, NULL);
	if (daemon != NULL)
		g_signal_emit_by_name (daemon, "folder_created", folder_info);
}

static void
folder_deleted_cb (DBusGProxy *proxy, GHashTable *folder_info, gpointer user_data)
{
	SyncdaemonDaemon *daemon = NULL;
	SyncdaemonFoldersInterface *interface = SYNCDAEMON_FOLDERS_INTERFACE (user_data);

	g_object_get (G_OBJECT (interface), "daemon", &daemon, NULL);
	if (daemon != NULL)
		g_signal_emit_by_name (daemon, "folder_deleted", folder_info);
}

static void
folder_subscribed_cb (DBusGProxy *proxy, GHashTable *folder_info, gpointer user_data)
{
	SyncdaemonDaemon *daemon = NULL;
	SyncdaemonFoldersInterface *interface = SYNCDAEMON_FOLDERS_INTERFACE (user_data);

	g_object_get (G_OBJECT (interface), "daemon", &daemon, NULL);
	if (daemon != NULL)
		g_signal_emit_by_name (daemon, "folder_subscribed", folder_info);
}

static void
folder_unsubscribed_cb (DBusGProxy *proxy, GHashTable *folder_info, gpointer user_data)
{
	SyncdaemonDaemon *daemon = NULL;
	SyncdaemonFoldersInterface *interface = SYNCDAEMON_FOLDERS_INTERFACE (user_data);

	g_object_get (G_OBJECT (interface), "daemon", &daemon, NULL);
	if (daemon != NULL)
		g_signal_emit_by_name (daemon, "folder_unsubscribed", folder_info);
}

static void
udf_error_cb (DBusGProxy *proxy, GHashTable *folder_info, gchar *error, gpointer user_data)
{
	SyncdaemonDaemon *daemon = NULL;
	SyncdaemonFoldersInterface *interface = SYNCDAEMON_FOLDERS_INTERFACE (user_data);

	g_object_get (G_OBJECT (interface), "daemon", &daemon, NULL);
	if (daemon != NULL)
		g_signal_emit_by_name (daemon, "error", error, folder_info);
}

static void
syncdaemon_folders_interface_init (SyncdaemonFoldersInterface *interface)
{
	interface->priv = g_new0 (SyncdaemonFoldersInterfacePrivate, 1);

	/* Setup DBus proxy */
	interface->priv->proxy = syncdaemon_interface_setup_proxy (SYNCDAEMON_INTERFACE (interface),
								   "com.ubuntuone.Syncdaemon",
								   "/folders", "com.ubuntuone.Syncdaemon.Folders");
	if (interface->priv->proxy != NULL) {
		/* Connect to DBus proxy signals */
		dbus_g_proxy_add_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderCreated",
					 dbus_g_type_get_map ("GHashTable",
							      G_TYPE_STRING,
							      G_TYPE_STRING),
					 G_TYPE_INVALID);
		dbus_g_proxy_connect_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderCreated",
					     G_CALLBACK (folder_created_cb), interface, NULL);

		dbus_g_proxy_add_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderDeleted",
					 dbus_g_type_get_map ("GHashTable",
							      G_TYPE_STRING,
							      G_TYPE_STRING),
					 G_TYPE_INVALID);
		dbus_g_proxy_connect_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderDeleted",
					     G_CALLBACK (folder_deleted_cb), interface, NULL);

		dbus_g_proxy_add_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderSubscribed",
					 dbus_g_type_get_map ("GHashTable",
							      G_TYPE_STRING,
							      G_TYPE_STRING),
					 G_TYPE_INVALID);
		dbus_g_proxy_connect_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderSubscribed",
					     G_CALLBACK (folder_subscribed_cb), interface, NULL);

		dbus_g_proxy_add_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderUnSubscribed",
					 dbus_g_type_get_map ("GHashTable",
							      G_TYPE_STRING,
							      G_TYPE_STRING),
					 G_TYPE_INVALID);
		dbus_g_proxy_connect_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderUnSubscribed",
					     G_CALLBACK (folder_unsubscribed_cb), interface, NULL);

		/* Error signals */
		dbus_g_proxy_add_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderCreateError",
					 dbus_g_type_get_map ("GHashTable",
							      G_TYPE_STRING,
							      G_TYPE_STRING),
					 G_TYPE_STRING,
					 G_TYPE_INVALID);
		dbus_g_proxy_connect_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderCreateError",
					     G_CALLBACK (udf_error_cb), interface, NULL);

		dbus_g_proxy_add_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderDeleteError",
					 dbus_g_type_get_map ("GHashTable",
							      G_TYPE_STRING,
							      G_TYPE_STRING),
					 G_TYPE_STRING,
					 G_TYPE_INVALID);
		dbus_g_proxy_connect_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderDeleteError",
					     G_CALLBACK (udf_error_cb), interface, NULL);

		dbus_g_proxy_add_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderSubscribeError",
					 dbus_g_type_get_map ("GHashTable",
							      G_TYPE_STRING,
							      G_TYPE_STRING),
					 G_TYPE_STRING,
					 G_TYPE_INVALID);
		dbus_g_proxy_connect_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderSubscribeError",
					     G_CALLBACK (udf_error_cb), interface, NULL);

		dbus_g_proxy_add_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderUnSubscribeError",
					 dbus_g_type_get_map ("GHashTable",
							      G_TYPE_STRING,
							      G_TYPE_STRING),
					 G_TYPE_STRING,
					 G_TYPE_INVALID);
		dbus_g_proxy_connect_signal (DBUS_G_PROXY (interface->priv->proxy), "FolderUnSubscribedError",
					     G_CALLBACK (udf_error_cb), interface, NULL);
	}
}

/**
 * syncdaemon_folders_interface_new:
 */
SyncdaemonFoldersInterface *
syncdaemon_folders_interface_new (SyncdaemonDaemon *daemon)
{
	g_return_val_if_fail (SYNCDAEMON_IS_DAEMON (daemon), NULL);

	return g_object_new (SYNCDAEMON_TYPE_FOLDERS_INTERFACE, "daemon", daemon, NULL);
}

/**
 * syncdaemon_folders_interface_create:
 */
void
syncdaemon_folders_interface_create (SyncdaemonFoldersInterface *interface, const gchar *path)
{
	g_return_if_fail (SYNCDAEMON_IS_FOLDERS_INTERFACE (interface));
	g_return_if_fail (path != NULL);

	dbus_g_proxy_begin_call (DBUS_G_PROXY (interface->priv->proxy), "create",
				 no_output_dbus_call_ended_cb, interface, NULL,
				 G_TYPE_STRING, path,
				 G_TYPE_INVALID);
}

/**
 * syncdaemon_folders_interface_delete:
 */
void
syncdaemon_folders_interface_delete (SyncdaemonFoldersInterface *interface, const gchar *folder_id)
{
	g_return_if_fail (SYNCDAEMON_IS_FOLDERS_INTERFACE (interface));
	g_return_if_fail (folder_id != NULL);

	dbus_g_proxy_begin_call (DBUS_G_PROXY (interface->priv->proxy), "delete",
				 no_output_dbus_call_ended_cb, interface, NULL,
				 G_TYPE_STRING, folder_id,
				 G_TYPE_INVALID);
}

/**
 * syncdaemon_folders_interface_get_folders:
 */
GSList *
syncdaemon_folders_interface_get_folders (SyncdaemonFoldersInterface *interface)
{
	GSList *folders = NULL;
	GError *error = NULL;

	g_return_val_if_fail (SYNCDAEMON_IS_FOLDERS_INTERFACE (interface), NULL);

	if (!dbus_g_proxy_call (DBUS_G_PROXY (interface->priv->proxy), "get_folders", &error,
				G_TYPE_INVALID,
				dbus_g_type_get_collection ("GSList",
							    dbus_g_type_get_map
							    ("GHashTable",
							     G_TYPE_STRING,
							     G_TYPE_STRING)), &folders,
				G_TYPE_INVALID)) {
		g_warning ("Error calling get_folders: %s", error->message);
		g_error_free (error);
	}

	return folders;
}

/**
 * syncdaemon_folders_interface_get_info:
 */
GHashTable *
syncdaemon_folders_interface_get_info (SyncdaemonFoldersInterface *interface, const gchar *path)
{
	GHashTable *info = NULL;
	GError *error = NULL;

	g_return_val_if_fail (SYNCDAEMON_IS_FOLDERS_INTERFACE (interface), NULL);
	g_return_val_if_fail (path != NULL, NULL);

	if (!dbus_g_proxy_call (DBUS_G_PROXY (interface->priv->proxy), "get_info", &error,
				G_TYPE_STRING, path,
				G_TYPE_INVALID,
				dbus_g_type_get_map ("GHashTable",
							G_TYPE_STRING,
							G_TYPE_STRING), &info,
				G_TYPE_INVALID)) {
		g_warning ("Error calling get_info: %s", error->message);
		g_error_free (error);
	}

	return info;
}

/**
 * syncdaemon_folders_interface_refresh_volumes:
 */
void
syncdaemon_folders_interface_refresh_volumes (SyncdaemonFoldersInterface *interface)
{
	g_return_if_fail (SYNCDAEMON_IS_FOLDERS_INTERFACE (interface));

	dbus_g_proxy_begin_call (DBUS_G_PROXY (interface->priv->proxy), "refresh_volumes",
				 no_output_dbus_call_ended_cb, interface, NULL,
				 G_TYPE_INVALID);
}

/**
 * syncdaemon_folders_interface_subscribe:
 */
void
syncdaemon_folders_interface_subscribe (SyncdaemonFoldersInterface *interface, const gchar *folder_id)
{
	g_return_if_fail (SYNCDAEMON_IS_FOLDERS_INTERFACE (interface));
	g_return_if_fail (folder_id != NULL);

	dbus_g_proxy_begin_call (DBUS_G_PROXY (interface->priv->proxy), "subscribe",
				 no_output_dbus_call_ended_cb, interface, NULL,
				 G_TYPE_STRING, folder_id,
				 G_TYPE_INVALID);
}

/**
 * syncdaemon_folders_interface_unsubscribe:
 */
void
syncdaemon_folders_interface_unsubscribe (SyncdaemonFoldersInterface *interface, const gchar *folder_id)
{
	g_return_if_fail (SYNCDAEMON_IS_FOLDERS_INTERFACE (interface));
	g_return_if_fail (folder_id != NULL);

	dbus_g_proxy_begin_call (DBUS_G_PROXY (interface->priv->proxy), "unsubscribe",
				 no_output_dbus_call_ended_cb, interface, NULL,
				 G_TYPE_STRING, folder_id,
				 G_TYPE_INVALID);
}
