/*
 * Syncdaemon API
 *
 * Authors: Rodrigo Moya <rodrigo.moya@canonical.com>
 *
 * Copyright 2010 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"
#include "syncdaemon-status-interface.h"
#include "utils.h"

G_DEFINE_TYPE(SyncdaemonStatusInterface, syncdaemon_status_interface, SYNCDAEMON_TYPE_INTERFACE)

struct _SyncdaemonStatusInterfacePrivate {
	GObject *proxy;
};

static void
syncdaemon_status_interface_finalize (GObject *object)
{
	SyncdaemonStatusInterface *interface = SYNCDAEMON_STATUS_INTERFACE (object);

	if (interface->priv != NULL) {
		g_free (interface->priv);
	}

	G_OBJECT_CLASS (syncdaemon_status_interface_parent_class)->finalize (object);
}

static void
syncdaemon_status_interface_class_init (SyncdaemonStatusInterfaceClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	object_class->finalize = syncdaemon_status_interface_finalize;
}

static void
download_started_cb (DBusGProxy *proxy, gchar *path, gpointer user_data)
{
	SyncdaemonDaemon *daemon;
	SyncdaemonStatusInterface *interface = SYNCDAEMON_STATUS_INTERFACE (user_data);

	g_object_get (G_OBJECT (interface), "daemon", &daemon, NULL);
	if (daemon != NULL)
		g_signal_emit_by_name (daemon, "download_started", path);
}

static void
download_finished_cb (DBusGProxy *proxy, gchar *path, GHashTable *info, gpointer user_data)
{
	SyncdaemonDaemon *daemon;
	SyncdaemonStatusInterface *interface = SYNCDAEMON_STATUS_INTERFACE (user_data);

	g_object_get (G_OBJECT (interface), "daemon", &daemon, NULL);
	if (daemon != NULL)
		g_signal_emit_by_name (daemon, "download_finished", path, info);
}

static void
upload_started_cb (DBusGProxy *proxy, gchar *path, gpointer user_data)
{
	SyncdaemonDaemon *daemon;
	SyncdaemonStatusInterface *interface = SYNCDAEMON_STATUS_INTERFACE (user_data);

	g_object_get (G_OBJECT (interface), "daemon", &daemon, NULL);
	if (daemon != NULL)
		g_signal_emit_by_name (daemon, "upload_started", path);
}

static void
upload_finished_cb (DBusGProxy *proxy, gchar *path, GHashTable *info, gpointer user_data)
{
	SyncdaemonDaemon *daemon;
	SyncdaemonStatusInterface *interface = SYNCDAEMON_STATUS_INTERFACE (user_data);

	g_object_get (G_OBJECT (interface), "daemon", &daemon, NULL);
	if (daemon != NULL)
		g_signal_emit_by_name (daemon, "upload_finished", path, info);
}

static void
syncdaemon_status_interface_init (SyncdaemonStatusInterface *interface)
{
	interface->priv = g_new0 (SyncdaemonStatusInterfacePrivate, 1);

	/* Setup DBus proxy */
	interface->priv->proxy = syncdaemon_interface_setup_proxy (SYNCDAEMON_INTERFACE (interface),
								   "com.ubuntuone.Syncdaemon",
								   "/status", "com.ubuntuone.Syncdaemon.Status");
	if (interface->priv->proxy != NULL) {
		/* Connect to DBus signals */
		dbus_g_proxy_add_signal (DBUS_G_PROXY (interface->priv->proxy), "DownloadStarted",
					 G_TYPE_STRING,
					 G_TYPE_INVALID);
		dbus_g_proxy_connect_signal (DBUS_G_PROXY (interface->priv->proxy), "DownloadStarted",
					     G_CALLBACK (download_started_cb), interface, NULL);

		dbus_g_proxy_add_signal (DBUS_G_PROXY (interface->priv->proxy), "DownloadFinished",
					 G_TYPE_STRING,
					 dbus_g_type_get_map ("GHashTable",
							      G_TYPE_STRING,
							      G_TYPE_STRING),
					 G_TYPE_INVALID);
		dbus_g_proxy_connect_signal (DBUS_G_PROXY (interface->priv->proxy), "DownloadFinished",
					     G_CALLBACK (download_finished_cb), interface, NULL);

		dbus_g_proxy_add_signal (DBUS_G_PROXY (interface->priv->proxy), "UploadStarted",
					 G_TYPE_STRING,
					 G_TYPE_INVALID);
		dbus_g_proxy_connect_signal (DBUS_G_PROXY (interface->priv->proxy), "UploadStarted",
					     G_CALLBACK (upload_started_cb), interface, NULL);

		dbus_g_proxy_add_signal (DBUS_G_PROXY (interface->priv->proxy), "UploadFinished",
					 G_TYPE_STRING,
					 dbus_g_type_get_map ("GHashTable",
							      G_TYPE_STRING,
							      G_TYPE_STRING),
					 G_TYPE_INVALID);
		dbus_g_proxy_connect_signal (DBUS_G_PROXY (interface->priv->proxy), "UploadFinished",
					     G_CALLBACK (upload_finished_cb), interface, NULL);
	}
}

/**
 * syncdaemon_status_interface_new:
 */
SyncdaemonStatusInterface *
syncdaemon_status_interface_new (SyncdaemonDaemon *daemon)
{
	g_return_val_if_fail (SYNCDAEMON_IS_DAEMON (daemon), NULL);

	return g_object_new (SYNCDAEMON_TYPE_STATUS_INTERFACE, "daemon", daemon, NULL);
}
